#!/usr/bin/env python

'''
Documentation, License etc.
'''

import argparse
import sys
import re
import xlsxwriter
import codecs
from datetime import datetime


def str_to_float(s):
    return float(s.replace('.','').replace(",",'.'))
    
def convert_pdf_to_txt(fp):
    import slate
    pdf = slate.PDF(fp)
    return pdf[0]

def parse_gutschrift(text):
    keys = {}

    a = re.search('Depotbestand[ ]*(?:Zinssatz[ ]*|)Wertpapier-Bezeichnung[ ]*WKN/ISIN'
                + 'per[ ](\d{2}.\d{2}.\d{4})[ ]*(\s{2,15}\d{1,2},\d{2}|)[ ]*(\S{6})[ ]*'  #datum,zinssatz & wkn
                + '((?:\S+[ ]{1})*)[ ]*'   #name
                + '(\S{3})[ ]*([\d,.]*)[ ]*([0-9A-Z]{12})[ ]*' #bestand (CUR+wert), isin
                + '((?:\S+[ ]{1})*)[ ]*'   #typ
                
                + '',text)
      
    if a!=None:
        text = text[a.end():] 
        keys['datum'] = datetime.strptime(a.groups()[0], "%d.%m.%Y")
        if len(a.groups()[1])>0:
           keys['zinssatz'] = str_to_float(a.groups()[1])
        keys['wkn'] = a.groups()[2]
        keys['name'] = a.groups()[3].rstrip(' ')
        keys['bestand_CUR'] = a.groups()[4]
        keys['bestand'] = str_to_float(a.groups()[5])
        keys['isin'] = a.groups()[6]
        keys['typ'] = a.groups()[7].rstrip(' ')
    
    a = re.search('Bruttobetrag:[ ]*(\S{3})[ ]*([\d,.]*)[  ]',text) #bruttobetrag (CUR+wert)
                
    if a!=None:
        text = text[a.end():] 
        keys['bruttobetrag_CUR'] = a.groups()[0]
        keys['bruttobetrag'] = str_to_float(a.groups()[1])
    
    a = re.search('Zu Ihren Gunsten vor Steuern[ ]*\S{0,2}[\d, ]*\S{3}[ ]*[\d]{0,8}[ ]*'
        + '[ ]*(\d{2}.\d{2}.\d{4})[ ]*(\S{3})[ ]*([\d,.]*)[ ]*' #valuta & gutschrift (CUR+wert)
        + '',text)
    
    if a!=None:
        keys['valuta'] = datetime.strptime(a.groups()[0], "%d.%m.%Y")
        keys['gutschrift_CUR'] = a.groups()[1]
        keys['gutschrift'] = str_to_float(a.groups()[2])
        
    return keys
    
    
def parse_abrechnung(text):
    
    keys = {}
    re.UNICODE
    a = re.search('Gesch\xc3\xa4ftstag[ ]*: (\d{2}\.\d{2}\.\d{4})',text)
    if a!=None:
        text = text[a.end():]
        keys['datum'] = datetime.strptime(a.groups()[0], "%d.%m.%Y")

    a = re.search('Wertpapier-Bezeichnung[ ]*WPKNR/ISIN[   ][ ]*'
                + '((?:\S*[ ]{1})*)[ ]*(\S{6})[   ][ ]*'         # name & wkn
                + '((?:\S+[ ]{1})*)[ ]*([0-9A-Z]{12})',text)     # typ  & isin
    
    if a!=None:
        text = text[a.end():] 
        keys['name'] = a.groups()[0].rstrip(' ')
        keys['wkn']  = a.groups()[1]
        keys['typ']  = a.groups()[2].rstrip(' ')
        keys['isin'] = a.groups()[3]
    
    a = re.search('Nennwert[ ]*Zum Kurs von[ ]*'
                + '(\S{3})[ ]*(\d+.\d*)[ ]*'  #nennwert (unit+wert),         
                + '(?:([A-Z]{3})[ ]*([\d.]+,\d{2})|([\d.]+,\d{2})(%))',text)   #kurs (curr+wert)
    
    if a!=None:
        text = text[a.end():]
        keys['nennwert_CUR'] = a.groups()[0]
        keys['nennwert']     = str_to_float(a.groups()[1])
        if a.groups()[5]=='%': #Anleihe in %
            keys['kurs_CUR'] = '%'
            keys['kurs']     = str_to_float(a.groups()[4])
        else:
            keys['kurs_CUR'] = a.groups()[2]
            keys['kurs']     = str_to_float(a.groups()[3])
        
    a = re.search('Kurswert[ ]*: ([A-Z]{3})[ ]*([\d.]+,\d{2})',text)
    if a!=None:
        text = text[a.end():]
        keys['kurswert_CUR'] = a.groups()[0]            
        keys['kurswert'] = str_to_float(a.groups()[1])
            
    a = re.search('\d{,3} Tage Zinsen[ ]*: ([A-Z]{3})[ ]*([\d.]+,\d{2})',text)
    if a!=None:
        text = text[a.end():]
        keys['stueckzinsen_CUR'] = a.groups()[0]
        keys['stueckzinsen'] = str_to_float(a.groups()[1])
    
    a = re.search('Provision[ ]*: ([A-Z]{3})[ ]*([\d.]+,\d{2})',text)
    if a!=None:
        text = text[a.end():]
        keys['provision_CUR'] = a.groups()[0]
        keys['provision'] = str_to_float(a.groups()[1])
        
    a = re.search('(B\xc3\xb6rsenplatzabh\xc3\xa4ng. Entgelt|Maklercourtage)[ ]*: ([A-Z]{3})[ ]*([\d.]+,\d{2})',text)
    if a!=None:
        text = text[a.end():]
        keys['entgelt_CUR'] = a.groups()[1]
        keys['entgelt'] = str_to_float(a.groups()[2])
        
    if text.find('Zu Ihren Gunsten vor Steuern')>0:
        if 'nennwert' in keys:
            keys['nennwert'] *= -1;
        if 'kurswert' in keys:
            keys['kurswert'] *= -1;
        if 'stueckzinsen' in keys:
            keys['stueckzinsen'] *= -1;
    return keys
        
def write_gutschrift(workbook,keys):
    worksheet = workbook.add_worksheet('Gutschriften')
    row = 0
    worksheet.write(row, 0, ur'Geschaeftstag')
    worksheet.write(row, 1, ur'Valuta')
    worksheet.write(row, 2, ur'Wertpapier')
    worksheet.write(row, 3, ur'WKN')
    worksheet.write(row, 4, ur'ISIN')
    worksheet.write(row, 5, ur'Bestand')
    worksheet.write(row, 6, ur'Zinssatz')
    worksheet.write(row, 7, ur'Gutschrift')
    worksheet.write(row, 8, ur'Bruttobetrag')
    for f in keys:
        row+=1
        if 'datum' in f:
            worksheet.write(row, 0, f['datum'], date_format)
        if 'valuta' in f:
            worksheet.write(row, 1, f['valuta'], date_format)
        if 'name' in f:
            worksheet.write(row, 2, f['name'])
        if 'wkn' in f:
            worksheet.write(row, 3, f['wkn'])
        if 'isin' in f:
            worksheet.write(row, 4, f['isin'])
        if 'bestand' in f:
            if f['bestand_CUR']=='EUR':
                worksheet.write(row, 5, f['bestand'], euro_format)
            elif f['bestand_CUR']=='USD':
                worksheet.write(row, 5, f['bestand'], dollar_format)
            else:
                worksheet.write(row, 5, f['bestand'])
            if 'zinssatz' in f:
                worksheet.write(row, 6, f['zinssatz']/100., percent_format)   
        if 'gutschrift' in f:
            if f['gutschrift_CUR']=='EUR':
                worksheet.write(row, 7, f['gutschrift'], euro_format)
            elif f['gutschrift_CUR']=='USD':
                worksheet.write(row, 7, f['gutschrift'], dollar_format)
            else:
                worksheet.write(row, 7, f['gutschrift'])
        if 'bruttobetrag' in f:
            if f['bruttobetrag_CUR']=='EUR':
                worksheet.write(row, 8, f['bruttobetrag'], euro_format)
            elif f['bruttobetrag_CUR']=='USD':
                worksheet.write(row, 8, f['bruttobetrag'], dollar_format)
            else:
                worksheet.write(row, 8, f['bruttobetrag'])
    
def write_abrechnung(workbook,keys):
    worksheet = workbook.add_worksheet('Abrechnungen')
    row = 0
    worksheet.write(row, 0, ur'Geschaeftstag')
    worksheet.write(row, 1, ur'Wertpapier')
    worksheet.write(row, 2, ur'WKN')
    worksheet.write(row, 3, ur'ISIN')
    worksheet.write(row, 4, ur'Nennwert')
    worksheet.write(row, 5, ur'Kurswert')
    worksheet.write(row, 6, ur'Stueckzinsen')
    worksheet.write(row, 7, ur'Kurs')
    worksheet.write(row, 8, ur'Provision')
    worksheet.write(row, 9, ur'Boersenentgelt')
    for f in keys:
        row+=1
        if 'datum' in f:
            worksheet.write(row, 0, f['datum'], date_format)
        if 'name' in f:
            worksheet.write(row, 1, f['name'])
        if 'wkn' in f:
            worksheet.write(row, 2, f['wkn'])
        if 'isin' in f:
            worksheet.write(row, 3, f['isin'])
        if 'nennwert' in f:
            if f['nennwert_CUR']=='EUR':
                worksheet.write(row, 4, f['nennwert'], euro_format)
            elif f['nennwert_CUR']=='USD':
                worksheet.write(row, 4, f['nennwert'], dollar_format)
            else:
                worksheet.write(row, 4, f['nennwert'])
        if 'kurswert' in f:
            if f['kurswert_CUR']=='EUR':
                worksheet.write(row, 5, f['kurswert'], euro_format)
            elif f['kurswert_CUR']=='USD':
                worksheet.write(row, 5, f['kurswert'], dollar_format)
            else:
                worksheet.write(row, 5, f['kurswert'])
        if 'stueckzinsen' in f:
            if f['stueckzinsen_CUR']=='EUR':
                worksheet.write(row, 6, f['stueckzinsen'], euro_format)
            elif f['stueckzinsen_CUR']=='USD':
                worksheet.write(row, 6, f['stueckzinsen'], dollar_format)
            else:
                worksheet.write(row, 6, f['stueckzinsen'])
        if 'kurs' in f:
            if f['kurs_CUR']=='EUR':
                worksheet.write(row, 7, f['kurs'], euro_format)
            elif f['kurs_CUR']=='USD':
                worksheet.write(row, 7, f['kurs'], dollar_format)
            else:
                worksheet.write(row, 7, f['kurs']/100., percent_format)
        if 'provision' in f:
            if f['provision_CUR']=='EUR':
                worksheet.write(row, 8, f['provision'], euro_format)
            elif f['provision_CUR']=='USD':
                worksheet.write(row, 8, f['provision'], dollar_format)
            else:
                worksheet.write(row, 8, f['provision'])
        if 'entgelt' in f:
            if f['entgelt_CUR']=='EUR':
                worksheet.write(row, 9, f['entgelt'], euro_format)
            elif f['entgelt_CUR']=='USD':
                worksheet.write(row, 9, f['entgelt'], dollar_format)
            else:
                worksheet.write(row, 9, f['entgelt'])
    
               
if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Dieses Skript liest Comdirect-Wertpapierabrechnungen und -Ertragsgutschriften im pdf-Format und speichert die wichtigsten Informationen in einer Excel-XLSX-Datei.')
    parser.add_argument('pdf',metavar='pdf',nargs='*',type=argparse.FileType('r'),help='Comdirect Wertpapiermeldung(en) im PDF-Format.')
    parser.add_argument('-o',dest='out',required=True,type=argparse.FileType('w'),help='Ausgabedatei (XLSX-Format). Vorsicht: Wenn Datei existiert wird sie \xc3\xbcberschrieben!')
    args = parser.parse_args()
    
    
    abrechnung = [];
    gutschrift = [];
    for f in args.pdf:
        text = convert_pdf_to_txt(f)
        if text.find('Gutschrift f\xc3\xa4lliger Wertpapier-Ertr\xc3\xa4ge')>0:
            gutschrift.append(parse_gutschrift(text))
        elif text.find('GESCH\xc3\x84FTSABRECHNUNG VOM ')>0:
            abrechnung.append(parse_abrechnung(text))
        else:
            print 'kenne die datei nicht'
    
    workbook       = xlsxwriter.Workbook(args.out.name)
    date_format    = workbook.add_format({'num_format': 'dd.mm.yyyy'})
    euro_format    = workbook.add_format({'num_format': ur'0.00 \xe2\x82\xac'})
    dollar_format  = workbook.add_format({'num_format': ur'$0.00'})
    percent_format = workbook.add_format({'num_format': ur'0.00%'})
    write_abrechnung(workbook,abrechnung)
    write_gutschrift(workbook,gutschrift)
    workbook.close()
    